package com.ahazou.photo_editor_sdk

import android.app.Activity
import android.app.Activity.RESULT_CANCELED
import android.app.Activity.RESULT_OK
import android.app.AlertDialog
import android.app.ProgressDialog
import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.net.Uri
import android.provider.MediaStore
import android.util.Log
import androidx.annotation.NonNull
import com.ahazou.photo_editor_sdk.service.ApiClient
import com.ahazou.photo_editor_sdk.service.ApiService
import com.ahazou.photo_editor_sdk.service.LogoToResponse
import com.bumptech.glide.Glide
import com.bumptech.glide.load.DataSource
import com.bumptech.glide.load.engine.GlideException
import com.bumptech.glide.request.RequestListener
import com.bumptech.glide.request.target.Target
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import ly.img.android.pesdk.PhotoEditorSettingsList
import ly.img.android.pesdk.assets.font.basic.FontPackBasic
import ly.img.android.pesdk.backend.decoder.ImageSource
import ly.img.android.pesdk.backend.layer.StickerGlLayer
import ly.img.android.pesdk.backend.layer.TextGlLayer
import ly.img.android.pesdk.backend.model.EditorSDKResult
import ly.img.android.pesdk.backend.model.config.FontAsset
import ly.img.android.pesdk.backend.model.config.ImageStickerAsset
import ly.img.android.pesdk.backend.model.constant.OutputMode
import ly.img.android.pesdk.backend.model.state.AssetConfig
import ly.img.android.pesdk.backend.model.state.LoadSettings
import ly.img.android.pesdk.backend.model.state.SaveSettings
import ly.img.android.pesdk.linker.ConfigMap
import ly.img.android.pesdk.ui.activity.PhotoEditorBuilder
import ly.img.android.pesdk.ui.model.state.UiConfigMainMenu
import ly.img.android.pesdk.ui.model.state.UiConfigSticker
import ly.img.android.pesdk.ui.model.state.UiConfigText
import ly.img.android.pesdk.ui.model.state.UiConfigTheme
import ly.img.android.pesdk.ui.panels.item.*
import ly.img.android.pesdk.ui.utils.DataSourceIdItemList
import ly.img.android.serializer._3.IMGLYFileReader
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import org.json.JSONObject
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import java.io.File
import java.io.FileOutputStream
import java.io.IOException
import java.lang.ref.WeakReference
import java.net.URL
import java.util.*

/** PhotoEditorSdkPlugin */
class PhotoEditorSdkPlugin :
        ActivityAware,
        FlutterPlugin,
        MethodCallHandler,
        PluginRegistry.ActivityResultListener {

    companion object {
        lateinit var activity: WeakReference<Activity>
        var arrLogos: ArrayList<CustomImageStickerAsset> = arrayListOf()
        var arrStickers: ArrayList<CustomImageStickerAsset> = arrayListOf()
        var userId = ""
        lateinit var resultClass: Result
        lateinit var progressDialog: ProgressDialog
        val SHARED_PREF = "sharedPreferences"
        private lateinit var context: Context
        const val PESDK_RESULT = 1
        const val GALLERY_RESULT = 2
    }

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        val channel = MethodChannel(flutterPluginBinding.binaryMessenger, "photo_editor_sdk")
        channel.setMethodCallHandler(PhotoEditorSdkPlugin())
    }

    override fun onDetachedFromActivity() {}

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {}

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = WeakReference(binding.activity)
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {}

    override fun onDetachedFromActivityForConfigChanges() {}

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        if (activity.get() != null) {
            context = activity.get()!!
        }
        resultClass = result
        when (call.method) {
            "getPlatformVersion" -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
            "addAllContents" -> {
                    val args = call.arguments as Map<*, *>
                val logos: ArrayList<*> = args["logos"] as ArrayList<*>
                val stickers: ArrayList<*> = args["stickers"] as ArrayList<*>
                arrLogos = retrieveAssets(logos)
                arrStickers = retrieveAssets(stickers)
            }
            "editImageByUrl" -> {
                val args = call.arguments as? String
                val uri = Uri.parse(args)
                openEditor(uri)
            }
            "editImageByFile" -> {
                val args = call.arguments as? String
                val file = File(args)
                val uri = Uri.fromFile(file)
                openEditor(uri)
            }
            "setUserId" -> {
                val id = call.arguments as String
                userId = id
            }
            "hasAddLogo" -> {
                val args = call.arguments as Map<*, *>
                if (args["hasAddLogo"] != null && args["urlAddLogo"] != null) {
                    val hasAddLogo = args["hasAddLogo"] as Boolean
                    val urlAddLogo = args["urlAddLogo"] as String

                    val editorShared = context.getSharedPreferences(SHARED_PREF, Context.MODE_PRIVATE).edit()
                    editorShared.putBoolean("hasAddLogo", hasAddLogo)
                    editorShared.putString("urlAddLogo", urlAddLogo)
                    editorShared.apply()
                }
            }
            "editImageSerialized" -> {
                val args = call.arguments as Map<*, *>
                val file = args["pathUrl"] as String
                val fileJson = args["jsonSerialized"] as String

                val uri = Uri.parse(file)
                openEditorSerialized(uri, fileJson)
            }
            "setEditorOptions" -> {
            }
            else -> result.notImplemented()
        }
    }

    private fun retrieveAssets(list: ArrayList<*>): ArrayList<CustomImageStickerAsset> {
        val result = ArrayList<CustomImageStickerAsset>()
        list.forEach {
            Glide
                    .with(activity.get()!!)
                    .asFile()
                    .load(it)
                    .listener(object : RequestListener<File> {
                        override fun onLoadFailed(e: GlideException?, model: Any?,
                                                  target: Target<File>?, isFirstResource: Boolean):
                                Boolean {
                            return false
                        }

                        override fun onResourceReady(resource: File?, model: Any?, target: Target<File>?,
                                                     dataSource: DataSource?, isFirstResource: Boolean):
                                Boolean {

                            val uri = Uri.fromFile(resource)

                            val imageStickerConfig = ImageStickerAsset.createTemporaryStickerAsset(uri)
                            result.add(CustomImageStickerAsset(imageStickerConfig, uri))
                            return false
                        }
                    })
                    .submit()
        }
        return result
    }


    private fun createPesdkSettingsList(inputImage: Uri?) =
            PhotoEditorSettingsList()

                    .configure<UiConfigText> {
                        val fontsInUiList = DataSourceIdItemList<FontItem>().apply {
                            add(FontItem("bungee", "Bungee"))
                            add(FontItem("cute_little_sheep", "Cute Little"))
                            add(FontItem("ubuntu", "Ubuntu"))
                            add(FontItem("sriracha", "Sriracha"))
                            add(FontItem("imgly_font_amaticsc", "Amaticsc"))
                            add(FontItem("imgly_font_imgly_font_archivo_black", "Archivo Black"))
                            add(FontItem("imgly_font_bungee_inline", "Bungee Inline"))
                            add(FontItem("imgly_font_campton_bold", "Campton Bold"))
                            add(FontItem("imgly_font_carter_one", "Carter One"))
                            add(FontItem("imgly_font_codystar", "Codystar"))
                            add(FontItem("imgly_font_fira_sans_regular", "Fira Sans"))
                            add(FontItem("imgly_font_galano_grotesque_bold", "Galano Grotesque"))
                            add(FontItem("imgly_font_krona_one", "Krona One"))
                            add(FontItem("imgly_font_kumar_one_outline", "Kumar One Outline"))
                            add(FontItem("imgly_font_lobster", "Lobster"))
                            add(FontItem("imgly_font_molle", "Molle"))
                            add(FontItem("imgly_font_monoton", "Monoton"))
                            add(FontItem("imgly_font_nixie_one", "Nixie One"))
                            add(FontItem("imgly_font_notable", "Notable"))
                            add(FontItem("imgly_font_open_sans_bold", "Open Sans Bold"))
                            add(FontItem("imgly_font_open_sans_bold", "Open Sans Bold"))
                            add(FontItem("imgly_font_ostrich_sans_black", "Ostrich Sans"))
                            add(FontItem("imgly_font_ostrich_sans_bold", "Ostrich Sans Bold"))
                            add(FontItem("imgly_font_oswald_semi_bold", "Oswald Semi Bold"))
                            add(FontItem("imgly_font_palanquin_dark_semi_bold", "Palanquin Dark"))
                            add(FontItem("imgly_font_permanent_marker", "Permanent Marker"))
                            add(FontItem("imgly_font_poppins", "Poppins"))
                            add(FontItem("imgly_font_roboto_black_italic", "Roboto Black"))
                            add(FontItem("imgly_font_roboto_light_italic", "Roboto Light"))
                            add(FontItem("imgly_font_sancreek", "Sancreek"))
                            add(FontItem("imgly_font_stint_ultra_expanded", "Stint Ultra"))
                            add(FontItem("imgly_font_trash_hand", "Trash Hand"))
                            add(FontItem("imgly_font_vt323", "Vt323"))
                            add(FontItem("imgly_font_yeseva_one", "Yeseva One"))
                            add(FontItem("imgly_font_aleo_bold", "Aleo Bold"))
                            add(FontItem("imgly_font_amberlight", "Amberlight"))
                            add(FontItem("imgly_font_archivo_black", "Archivo Black"))
                            add(FontItem("imgly_font_handycheera_regular", "Handycheera Regular"))
                            add(FontItem("imgly_font_montserrat_light", "Montserrat Light"))
                            add(FontItem("imgly_font_ostrich_sans_heavy", "Ostrich Sans Heavy"))
                            add(FontItem("imgly_font_petit_formal_script", "Petit Formal"))
                            add(FontItem("imgly_font_rasa_500", "Rasa"))
                            add(FontItem("imgly_font_rasa_regular", "Rasa Regular"))
                            add(FontItem("imgly_font_sue_ellen_francisco", "Sue Ellen"))
                            add(FontItem("imgly_font_ultra", "Ultra"))
                            add(FontItem("imgly_font_wolesbro", "Wolesbro"))
                            add(FontItem("imgly_font_abril_fatface_regular", "Abril Fatface"))
                        }

                        it.setFontList(fontsInUiList)
                    }
                    .configure<SaveSettings> {
                        it.setOutputToTemp()
                        it.outputMode = OutputMode.EXPORT_ALWAYS
                    }
                    .configure<LoadSettings> {
                        it.source = inputImage
                    }
                    .configure<UiConfigTheme> {
                        it.theme = R.style.Imgly_Theme_TopActionBar_NoFullscreen
                    }
                    .configure<UiConfigMainMenu> {
                        it.setToolList(
                                ToolItem("imgly_tool_sticker_selection",
                                        R.string.pesdk_sticker_title_name,
                                        ImageSource.create(R.drawable.imgly_icon_tool_sticker)),
                                ToolItem("imgly_tool_text",
                                        R.string.pesdk_text_title_name,
                                        ImageSource.create(R.drawable.imgly_icon_tool_text))
                        )
                    }
                    .configure<UiConfigSticker> {

                        StickerGlLayer.SORTED_ROTATION_SNAP_POINTS = FloatArray(6)
                        StickerGlLayer.SNAP_PADDING_LEFT = 0.0f
                        StickerGlLayer.SNAP_PADDING_TOP = 0.0f
                        StickerGlLayer.SNAP_PADDING_RIGHT = 0.0f
                        StickerGlLayer.SNAP_PADDING_BOTTOM = 0.0f

                        TextGlLayer.SORTED_ROTATION_SNAP_POINTS = FloatArray(6)
                        TextGlLayer.SNAP_PADDING_LEFT = 0.0f
                        TextGlLayer.SNAP_PADDING_TOP = 0.0f
                        TextGlLayer.SNAP_PADDING_RIGHT = 0.0f
                        TextGlLayer.SNAP_PADDING_BOTTOM = 0.0f


                        val arrLogoSticker = ArrayList<AbstractIdItem>()

                        // Add-logo button
                        if (context.getSharedPreferences(SHARED_PREF, Context.MODE_PRIVATE).getBoolean("hasAddLogo", false)) {
                            arrLogoSticker.add(PersonalStickerAddItem("imgly_add_personal_sticker",
                                    R.string.pesdk_sticker_button_add,
                                    ImageSource.create(R.drawable.ic_add_logo)))
                        }

                        // Stickers & logos
                        if (arrStickers.isNotEmpty()) {
                            arrLogoSticker.add(StickerCategoryItem(
                                    "stickers_id",
                                    "stickers",
                                    ImageSource.create(R.mipmap.ic_sticker),
                                    arrStickers.map { s -> ImageStickerItem.createFromAsset(s.image) }
                            ))
                            arrLogoSticker.add(StickerCategoryItem(
                                    "imgly_add_personal_sticker",
                                    "logos",
                                    ImageSource.create(R.mipmap.ic_logo),
                                    arrLogos.map { s -> ImageStickerItem.createFromAsset(s.image) }
                            ))
                        }
                        it.setStickerLists(arrLogoSticker)
                    }
                    .configure<AssetConfig> {
                        it.addAsset()


                        arrLogos.map { logo -> it.addAsset(logo.image) }
                        arrStickers.map { sticker -> it.addAsset(sticker.image) }
                    }

    val PESDK_RESULT = 1000

    private fun openEditor(inputImage: Uri?) {
        val settingsList = createPesdkSettingsList(inputImage)

        Thread {
            PhotoEditorBuilder(activity.get())
                    .setSettingsList(settingsList)
                    .startActivityForResult(activity.get()!!, PESDK_RESULT)
        }.run()
    }

    private fun openEditorSerialized(inputImage: Uri?, jsonUrl: String?) {
        try {
            try {
                val thread = Thread{
                    val settingsList = createPesdkSettingsList(inputImage)
                    settingsList.getSettingsModel(AssetConfig::class.java)!!.getAssetMap(FontAsset::class.java).apply {
                        add(FontAsset("bungee", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/bungee.ttf")))
                        add(FontAsset("cute_little_sheep", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/cute_little_sheep.ttf")))
                        add(FontAsset("ubuntu", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/ubuntu.ttf")))
                        add(FontAsset("sriracha", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/sriracha.ttf")))
                        add(FontAsset( "imgly_font_amberlight", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_amberlight.ttf")))
                        add(FontAsset( "imgly_font_archivo_black", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_archivo_black.ttf")))
                        add(FontAsset( "imgly_font_handycheera_regular", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_handycheera_regular.ttf")))
                        add(FontAsset( "imgly_font_montserrat_light", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_montserrat_light.ttf")))
                        add(FontAsset( "imgly_font_ostrich_sans_heavy", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_ostrich_sans_heavy.ttf")))
                        add(FontAsset( "imgly_font_petit_formal_script", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_petit_formal_script.ttf")))
                        add(FontAsset( "imgly_font_rasa_500", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_rasa_500.ttf")))
                        add(FontAsset( "imgly_font_rasa_regular", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_rasa_regular.ttf")))
                        add(FontAsset( "imgly_font_sue_ellen_francisco", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_sue_ellen_francisco.ttf")))
                        add(FontAsset( "imgly_font_ultra", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_ultra.ttf")))
                        add(FontAsset( "imgly_font_wolesbro", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_wolesbro.ttf")))
                        add(FontAsset( "imgly_font_abril_fatface_regular", Uri.parse("https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_abril_fatface_regular.ttf")))
                    }


                    val reader = IMGLYFileReader(settingsList)

                    val stringJson = URL(jsonUrl).readText()
                    var obj = JSONObject(stringJson)
                    reader.readJson(obj.toString())

                    PhotoEditorBuilder(activity.get())
                            .setSettingsList(settingsList)
                            .startActivityForResult(activity.get()!!, PESDK_RESULT)
                }
                thread.start()
            } catch (e: java.lang.Exception) {
                e.printStackTrace()
            }

        } catch (e: java.lang.IllegalArgumentException) {
            e.printStackTrace()
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
        if (resultCode == RESULT_OK && requestCode == GALLERY_RESULT) {
            // Open Editor with some uri in this case with an image selected from the system gallery.
            val selectedImage = data?.data
            if (selectedImage != null) {
                openEditor(selectedImage)
            }
        } else if (resultCode == RESULT_OK && requestCode == PESDK_RESULT) {
            // Editor has saved an Image.
            val data = data?.let { EditorSDKResult(it) }

            if (data != null) {
                Log.i("PESDK", "Source image is located here ${data.sourceUri}")
                Log.i("PESDK", "Result image is located here ${data.resultUri}")
            }

            val lastState = data?.settingsList


            try {
                val model = lastState?.getSettingsModel(AssetConfig::class.java)
                if (model != null) {
                    val mapAsset = model!!.getAssetMap(ImageStickerAsset::class.java)
                    val arrLogosSticker = ArrayList<ImageStickerAsset>()
                    arrLogos.map { logos -> arrLogosSticker.add(logos.image) }
                    arrStickers.map { logos -> arrLogosSticker.add(logos.image) }

                    val dynamicStickers = ArrayList<ImageStickerAsset>()
                    mapAsset.map { map -> dynamicStickers.add(map) }

                    print(dynamicStickers.size);
                    if (dynamicStickers.size > arrLogosSticker.size) {
                        val newLogo = ArrayList<ImageStickerAsset>()

                        for (new in dynamicStickers) {
                            var found = false
                            for (logo in arrLogosSticker) {
                                if (new.id == logo.id) {
                                    found = true
                                    break
                                }
                            }
                            if (!found) newLogo.add(new)
                        }

                        if (newLogo.size == 1) {
                            if (activity.get() != null) {
                                if (!activity.get()!!.isFinishing) {
                                    progressDialog = ProgressDialog(activity.get()!!)
                                    progressDialog.setMessage("Enviando dados...")
                                    progressDialog.setCancelable(false)
                                    progressDialog.show()
                                }
                            }
                            newLogo.forEachIndexed { i, logo ->
                                Thread {
                                    if (logo.stickerSource.uri != null) {
                                        val bitmap: Bitmap = MediaStore.Images.Media.getBitmap(activity.get()?.contentResolver, logo.stickerSource.uri)

                                        val resized = resize(bitmap, 300, 300)

                                        val persistedLogo = persistLogo(resized)
                                        if (persistedLogo != null) {
                                            data.resultUri?.path?.let {
                                                uploadLogo(persistedLogo, i == newLogo.size - 1, it)
                                            }

                                        }
                                    }
                                }.start()
                            }
                        } else {
                            data.resultUri?.path?.let { savePost(it) }
                        }
                    } else {
                        data.resultUri?.path?.let { savePost(it) }
                    }
                }
            } catch (e: IOException) {
                e.printStackTrace()
            }
            return true

        } else if (resultCode == RESULT_CANCELED && requestCode == PESDK_RESULT) {
            // Editor was canceled
            val data = data?.let { EditorSDKResult(it) }

            val sourceURI = data?.sourceUri
            // TODO: Do something with the source...*/
            return false
        }
        return false
    }

    private fun resize(image: Bitmap, maxWidth: Int, maxHeight: Int): Bitmap {
        val image: Bitmap = image
        if (maxHeight > 0 && maxWidth > 0) {
            val width: Int = image.getWidth()
            val height: Int = image.getHeight()
            val ratioBitmap = width.toFloat() / height.toFloat()
            val ratioMax = maxWidth.toFloat() / maxHeight.toFloat()
            var finalWidth = maxWidth
            var finalHeight = maxHeight
            if (ratioMax > ratioBitmap) {
                finalWidth = (maxHeight.toFloat() * ratioBitmap).toInt()
            } else {
                finalHeight = (maxWidth.toFloat() / ratioBitmap).toInt()
            }
            return Bitmap.createScaledBitmap(image, finalWidth, finalHeight, true)
        } else {
            return image
        }
    }

    private fun savePost(path: String) {
        val file = File(path)
        val b = file.readBytes()
        resultClass.success(b)
    }

    private fun persistLogo(bitmap: Bitmap, name: String = "temporary_logo"): File? {
        if (activity.get() != null) {

            val filesDir = activity.get()!!.applicationContext.filesDir
            val imageFile = File(filesDir, "$name.png")

            try {
                val os = FileOutputStream(imageFile)
                bitmap.compress(Bitmap.CompressFormat.PNG, 100, os)
                os.flush()
                os.close()
            } catch (e: Exception) {
                Log.e(this.javaClass.simpleName, "Error writing bitmap", e)
            }
            return imageFile
        }
        return null
    }

    private fun uploadLogo(file: File, lastOne: Boolean, path: String) {
        val requestBody = RequestBody.create(MediaType.parse("*/*"), file)
        val fileToUpload = MultipartBody.Part.createFormData("logo", file.name, requestBody)
        var multipartUserId = MultipartBody.Part.createFormData("userId", userId)

        ApiClient.BASE_URL = context.getSharedPreferences(SHARED_PREF, Context.MODE_PRIVATE).getString("urlAddLogo", "")
        val getResponse = ApiClient.getClient().create(ApiService::class.java)


        val call = getResponse.addLogo(multipartUserId, fileToUpload)

        call.enqueue(object : Callback<LogoToResponse> {
            override fun onFailure(call: Call<LogoToResponse>, t: Throwable) {
                progressDialog.dismiss()
                if (activity.get() != null) {
                    val builder = AlertDialog.Builder(activity.get()!!, AlertDialog.THEME_DEVICE_DEFAULT_LIGHT)
                    builder.setTitle("ops!")
                    builder.setMessage("não conseguimos salvar sua logo. quer tentar novamente?")
                    builder.setPositiveButton("ok") { dialog, which -> if (lastOne) savePost(path) }
                    val dialog: AlertDialog = builder.create()

                    dialog.show()
                }
            }

            override fun onResponse(call: Call<LogoToResponse>, response: Response<LogoToResponse>) {
                progressDialog.dismiss()

                if (lastOne) savePost(path)
            }
        })
    }

    data class CustomImageStickerAsset(val image: ImageStickerAsset, val uri: Uri)
}