import Flutter
import PhotoEditorSDK
import Foundation

protocol MyPhotoEditorDelegate {
    func isLogoStickerTool(_ value: Bool)
}

public class MyPhotoEditor: PhotoEditViewController {
    let podBundle = Bundle(for: SwiftPhotoEditorSdkPlugin.self) // for getting pod url
    var myDelegate: MyPhotoEditorDelegate!
    public override func viewDidLoad() {
        super.viewDidLoad()
        
    }
    
    private static let defaultTheme: Theme = {
        return Theme(backgroundColor: Colors.neutral50, menuBackgroundColor: .white, toolbarBackgroundColor: .white, primaryColor: Colors.primmary)
    }()
    
    private var theme = defaultTheme
    
    public override func menuViewController(_ menuViewController: MenuViewController, didSelect menuItem: MenuItem) {
        let teste = menuItem as? ToolMenuItem
        if teste?.title == "logos" {
            myDelegate.isLogoStickerTool(true)
        } else {
            myDelegate.isLogoStickerTool(false)
        }
        super.menuViewController(menuViewController, didSelect: menuItem)
    }
}

public class SwiftPhotoEditorSdkPlugin: NSObject, FlutterPlugin, MyPhotoEditorDelegate {
    var controller: UIViewController!
    var globalResult: FlutterResult!
    let podBundle = Bundle(for: SwiftPhotoEditorSdkPlugin.self) // for getting pod url
    private var menuItems = [PhotoEditMenuItem]()
    private var isLogoStickerTool = false
    static var registrarToSendMessages : FlutterPluginRegistrar? = nil
    
    func isLogoStickerTool(_ value: Bool) {
        isLogoStickerTool = value
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        self.translate()
        if let licenseURL = Bundle.main.url(forResource: "license", withExtension: "") {
            PESDK.unlockWithLicense(at: licenseURL)
        }
        let channel = FlutterMethodChannel(name: "photo_editor_sdk", binaryMessenger: registrar.messenger())
        let instance = SwiftPhotoEditorSdkPlugin()
        registrarToSendMessages = registrar
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        controller = UIApplication.shared.keyWindow?.rootViewController
        globalResult = result
        let methodIdentifier = MethodIdentifierEnum(rawValue: call.method)
        switch(methodIdentifier) {
        case .hasAddLogo:
            if let args = call.arguments as? Dictionary<String, Any> {
                guard let hasAddLogo = args["hasAddLogo"] as? String,
                      let urlAddLogo = args["urlAddLogo"] as? String else {
                    return
                }
                UserDefaults.standard.set(hasAddLogo, forKey: "hasAddLogo")
                UserDefaults.standard.set(urlAddLogo, forKey: "urlAddLogo")
            }
            break
        case .addAllContents:
            if let args = call.arguments as? Dictionary<String, Array<String>> {
                UserDefaults.standard.set(nil, forKey: "logos")
                UserDefaults.standard.set(nil, forKey: "stickers")
                UserDefaults.standard.set(args["logos"], forKey: "logos")
                UserDefaults.standard.set(args["stickers"], forKey: "stickers")
                setUpStickers()
            }
            break
        case .editImageSerialized:
            if let args = call.arguments as? Dictionary<String, String> {
                guard let image = args["pathUrl"],
                    let url = URL(string: image) else { return }
                do {
                    let data = try Data(contentsOf: url)
                    if let uiImage = UIImage(data: data) {
                        let photoImage = Photo(image: uiImage)
                        if let jsonSerr = args["jsonSerialized"]{
                            readSerialized(path: jsonSerr, photo: photoImage)
                        }
                    }
                } catch {
                    print(error)
                }
            }
            break
        case .editImageByUrl:
            guard let args = call.arguments as? String,
                let url = URL(string: args)
                else {
                    return
            }
            do {
                let data = try Data(contentsOf: url)
                if let uiImage = UIImage(data: data) {
                    let photoImage = Photo(image: uiImage)
                    self.presentPhotoEditViewController(photo: photoImage)
                }
            } catch {
                print(error)
            }
            break
        case .editImageByFile:
            guard let args = call.arguments as? String,
                let image = UIImage(named: args) else { return }
            let photoImage = Photo(image: image)
            self.presentPhotoEditViewController(photo: photoImage)
            break
        case .setUserId:
            let args = call.arguments as? String
            UserDefaults.standard.set(args, forKey: "userId")
        case .setEditorOptions:
            if let args = call.arguments as? Array<String> {
                createDefaultItems(selecteds: args)
            }
        default:
            break
        }
    }
    
    func readSerialized(path: String, photo: Photo){
        if let url = URL(string: path){
            URLSession.shared.dataTask(with: url) { (data, response, error) -> Void in
                if let dataNotNull = data, error == nil {
                    do {
                        let json = try JSONSerialization.jsonObject(with: dataNotNull, options: .mutableContainers) as? [String: Any]
                        self.presentPhotoEditViewControllerSerialized(photo: photo, json: dataNotNull)
                    } catch {
                        print("Unexpected error reading file.")
                    }
                }
            }.resume()
        }
    }
    private func presentPhotoEditViewControllerSerialized(photo: Photo, json: Data) {
        let inputImage: Photo = photo
        let data: Data = json
        let photo = inputImage
        let assetCatalog = AssetCatalog.sharedItems
        let deserializationResult = Deserializer.deserialize(data: data, imageDimensions: inputImage.size, assetCatalog: assetCatalog)
        if let model = deserializationResult.model {
            let configuration = buildConfiguration()
            let photoEditViewController = MyPhotoEditor(photoAsset: photo, configuration: configuration, photoEditModel: model)
            photoEditViewController.myDelegate = self
            photoEditViewController.modalPresentationStyle = .fullScreen
            photoEditViewController.delegate = self
            DispatchQueue.main.async {
                self.controller.present(self.createPhotoEditViewController(with: photo, and: model), animated: true, completion: nil)
            }
        }
        
    }
    
    private func presentPhotoEditViewController(photo: Photo) {
        DispatchQueue.main.async {
            self.controller.present(self.createPhotoEditViewController(with: photo), animated: true, completion: nil)
        }
    }
    
    private func createPhotoEditViewController(with photo: Photo, and photoEditModel: PhotoEditModel = PhotoEditModel()) -> PhotoEditViewController {
        let configuration = buildConfiguration()
        
        // Create a photo edit view controller
        let photoEditViewController = MyPhotoEditor(photoAsset: photo, configuration: configuration, photoEditModel: photoEditModel)
        photoEditViewController.myDelegate = self
        photoEditViewController.modalPresentationStyle = .fullScreen
        photoEditViewController.delegate = self
        
        
        return photoEditViewController
    }
    
    func createDefaultItems(selecteds: Array<String>) {
        guard let imageLogo = UIImage(named: "ico_logo", in: podBundle, compatibleWith: nil),
            let imageStickers = UIImage(named: "ico_sticker", in: podBundle, compatibleWith: nil),
            let imageFilter = UIImage(named: "ico_filter", in: podBundle, compatibleWith: nil)
            else {
                return
        }
        
        selecteds.forEach { (option) in
            let optionEnum = PhotoEditMenuEnum(rawValue: option)
            switch optionEnum {
                
            case .brush:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createBrushToolItem()!))
                break
            case .frame:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createFrameToolItem()!))
                break
            case .transform:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createTransformToolItem()!))
                break
            case .logo:
                let logoTool = ToolMenuItem(
                    title: "logos",
                    icon: imageLogo,
                    toolControllerClass: LogoStickerToolController.self)!
                menuItems.append(PhotoEditMenuItem.tool(logoTool))
                break
            case .sticker:
                let adesivosTool = PhotoEditMenuItem.tool(
                    ToolMenuItem(
                        title: "adesivos",
                        icon: imageStickers,
                        toolControllerClass: CustomStickerController.self)!
                )
                menuItems.append(adesivosTool)
                break
            case .text:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createTextToolItem()!))
                break
            case .textDesign:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createTextDesignToolItem()!))
                break
            case .adjust:
                menuItems.append(PhotoEditMenuItem.tool(ToolMenuItem.createAdjustToolItem()!))
                break
            case .filter:
                let filterTool = PhotoEditMenuItem.tool(
                    ToolMenuItem(
                        title: "filtro",
                        icon: imageFilter,
                        toolControllerClass: CustomFilterToolController.self)!
                )
                menuItems.append(filterTool)
                break
            case .none:
                break
            }
        }
    }
    
    private func configureTransformToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureTransformToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "transformar")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureFrameToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureFrameToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "moldura")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureFrameOptionsToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "quadro")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureBrushToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureBrushToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "pincel")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureBrushColorToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "pincel")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureStickerToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureStickerToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: self.isLogoStickerTool ? "logos" : "adesivos")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureStickerOptionsToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureTextToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureTextToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "editor")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureTextFontToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureTextColorToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureTextOptionsToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureAdjustToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureAdjustToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureFilterToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureFilterToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func configureDesignTextToolbar(builder: (ImglyKit.ConfigurationBuilder)) {
        builder.configureTextDesignToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
        
        builder.configureTextDesignOptionsToolController { (options) in
            options.discardButtonConfigurationClosure = { button in
                button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
            }
            
            options.titleViewConfigurationClosure = { view in
                let label = self.createTitleToolbarLabel(title: "opções")
                view.addSubview(label)
                self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
            }
        }
    }
    
    private func buildConfiguration() -> Configuration {
        let configuration = Configuration { builder in
            // Configure editor
            builder.configurePhotoEditViewController { options in
                options.menuItems = menuItems
                
                options.titleViewConfigurationClosure = { view in
                    /// This will add and UILabel upside of title of the toolBar. Creating a layout more trustworthy with Ahazou.
                    let label = self.createTitleToolbarLabel(title: "editor")
                    view.addSubview(label)
                    self.applyConstraintsOnTitleToolbarLabel(label: label, view: view)
                }
                
                options.discardButtonConfigurationClosure = { button in
                    button.setImage(UIImage(named: "close", in: self.podBundle, compatibleWith: nil), for: .normal)
                }
                
                options.applyButtonConfigurationClosure = { button in
                    /// Editing the download/save button of PhotoEditView
                    button.setImage(UIImage(), for: .normal)
                    button.setTitle("salvar", for: .normal)
                    button.titleLabel?.font = UIFont.systemFont(ofSize: 18, weight: .semibold)
                    button.titleEdgeInsets.right = 20
                    button.titleEdgeInsets.left = -20
                }
            }
            
            configureTextToolbar(builder: builder)
            configureStickerToolbar(builder: builder)
            configureBrushToolbar(builder: builder)
            configureFrameToolbar(builder: builder)
            configureTransformToolbar(builder: builder)
            configureAdjustToolbar(builder: builder)
            configureFilterToolbar(builder: builder)
            configureDesignTextToolbar(builder: builder)
            
            // Configure theme
            builder.theme = self.theme
            customizeFonts()
        }
        
        return configuration
    }

    private func loadFont(bundle: Bundle = Bundle(for: SwiftPhotoEditorSdkPlugin.self), withName fontName: String, ext: String = "ttf") {
        guard
          let fontURL = bundle.url(forResource: fontName, withExtension: "ttf"),
          let fontData = try? Data(contentsOf: fontURL) as CFData,
          let provider = CGDataProvider(data: fontData),
          let font = CGFont(provider) else {
            loadFontOtf(bundle: bundle, withName: fontName)
            return
        }
        CTFontManagerRegisterGraphicsFont(font, nil)
    }
    
    private func loadFontOtf(bundle: Bundle = Bundle(for: SwiftPhotoEditorSdkPlugin.self), withName fontName: String) {
        guard
          let fontURL = bundle.url(forResource: fontName, withExtension: "otf"),
          let fontData = try? Data(contentsOf: fontURL) as CFData,
          let provider = CGDataProvider(data: fontData),
          let font = CGFont(provider) else {
            return
        }
        CTFontManagerRegisterGraphicsFont(font, nil)
    }
    
    private func customizeFonts() {
        // TO-DO Add font names to a JSON or something to iterate.
        
        loadFont(bundle: Bundle.imglyBundle ,withName: "imgly_font_amaticsc")
        loadFont(bundle: Bundle.imglyBundle ,withName: "imgly_font_aleo_bold")
        loadFont(withName: "imgly_font_carter_one")
        loadFont(withName: "imgly_font_codystar")
        loadFont(withName: "imgly_font_yeseva_one")
        loadFont(withName: "imgly_font_vt323")
        loadFont(withName: "imgly_font_trash_hand")
        loadFont(withName: "imgly_font_stint_ultra_expanded")
        loadFont(withName: "imgly_font_sancreek")
        loadFont(withName: "imgly_font_roboto_light_italic")
        loadFont(withName: "imgly_font_roboto_black_italic")
        loadFont(withName: "imgly_font_poppins")
        loadFont(withName: "imgly_font_palanquin_dark_semi_bold")
        loadFont(withName: "imgly_font_oswald_semi_bold")
        loadFont(withName: "imgly_font_lobster")
        loadFont(withName: "imgly_font_notable")
        loadFont(withName: "imgly_font_nixie_one")
        loadFont(withName: "imgly_font_molle")
        loadFont(withName: "imgly_font_monoton")
        loadFont(withName: "imgly_font_kumar_one_outline")
        loadFont(withName: "imgly_font_krona_one")
        loadFont(withName: "imgly_font_fira_sans_regular")
 
        FontImporter.all = [
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_abril_fatface_regular.ttf",
                     name: "Abril Fatface",
                     identifier: "imgly_font_abril_fatface_regular"),
            Font(displayName: "Amatic", fontName: "AmaticSC-Bold", identifier: "imgly_font_amaticsc"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_amberlight.ttf",
                     name: "Amberlight",
                     identifier: "imgly_font_amberlight"),
            Font(displayName: "Aleo Bold", fontName: "Aleo-Bold", identifier: "imgly_font_aleo_bold"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_archivo_black.ttf",
                     name: "Archivo Black",
                     identifier: "imgly_font_archivo_black"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/bungee.ttf",
                     name: "Bungee",
                     identifier: "bungee"),
            Font(displayName: "Carter One", fontName: "CarterOne", identifier: "imgly_font_carter_one"),
            Font(displayName: "Bungee Inline", fontName: "Bungee Inline", identifier: "imgly_font_bungee_inline"),
            Font(displayName: "Campton Bold", fontName: "Campton Bold", identifier: "imgly_font_campton_bold"),
            Font(displayName: "Codystar", fontName: "Codystar", identifier: "imgly_font_codystar"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/cute_little_sheep.ttf",
                     name: "Cute Little Sheep",
                     identifier: "cute_little_sheep"),
            Font(displayName: "Fira Sans", fontName: "FiraSans-Regular", identifier: "imgly_font_fira_sans_regular"),
            Font(displayName: "Galano Grotesque", fontName: "Galano Grotesque", identifier: "imgly_font_galano_grotesque_bold"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_handycheera_regular.ttf",
                     name: "Handycheera Regular",
                     identifier: "imgly_font_handycheera_regular"),
            Font(displayName: "Krona One", fontName: "KronaOne-Regular", identifier: "imgly_font_krona_one"),
            Font(displayName: "Kumar One Outline", fontName: "KumarOneOutline-Regular", identifier: "imgly_font_kumar_one_outline"),
            Font(displayName: "Monoton", fontName: "Monoton-Regular", identifier: "imgly_font_monoton"),
            Font(displayName: "Molle", fontName: "Molle-Regular", identifier: "imgly_font_molle"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_montserrat_light.ttf",
                     name: "Montserrat Light",
                     identifier: "imgly_font_montserrat_light"),
            Font(displayName: "Nixie One", fontName: "NixieOne-Regular", identifier: "imgly_font_nixie_one"),
            Font(displayName: "Notable", fontName: "Notable-Regular", identifier: "imgly_font_notable"),
            Font(displayName: "Lobster", fontName: "Lobster-Regular", identifier: "imgly_font_lobster"),
            Font(displayName: "Open Sans", fontName: "Open Sans", identifier: "imgly_font_open_sans_bold"),
            Font(displayName: "Ostrich Sans", fontName: "Ostrich Sans", identifier: "imgly_font_ostrich_sans_black"),
            Font(displayName: "Ostrich Sans Bold", fontName: "Ostrich Sans Bold", identifier: "imgly_font_ostrich_sans_bold"),
            Font(displayName: "Oswald Semi Bold", fontName: "Oswald-SemiBold", identifier: "imgly_font_oswald_semi_bold"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_ostrich_sans_heavy.ttf",
                     name: "Ostrich Sans Heavy",
                     identifier: "imgly_font_ostrich_sans_heavy"),
            Font(displayName: "Palanquin Dark", fontName: "PalanquinDark-SemiBold", identifier: "imgly_font_palanquin_dark_semi_bold"),
            Font(displayName: "Permanent Marker", fontName: "Permanent Marker", identifier: "imgly_font_permanent_marker"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_petit_formal_script.ttf",
                     name: "Petit Formal",
                     identifier: "imgly_font_petit_formal_script"),
            Font(displayName: "Poppins", fontName: "Poppins-Regular", identifier: "imgly_font_poppins"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_rasa_500.ttf",
                     name: "Rasa",
                     identifier: "imgly_font_rasa_500"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_rasa_regular.ttf",
                     name: "Rasa Regular",
                     identifier: "imgly_font_rasa_regular"),
            Font(displayName: "Roboto Black", fontName: "Roboto-BlackItalic", identifier: "imgly_font_roboto_black_italic"),
            Font(displayName: "Roboto Light", fontName: "Roboto-LightItalic", identifier: "imgly_font_roboto_light_italic"),
            Font(displayName: "Sancreek", fontName: "Sancreek-Regular", identifier: "imgly_font_sancreek"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/sriracha.ttf",
                     name: "Sriracha",
                     identifier: "sriracha"),
            Font(displayName: "Stint Ultra", fontName: "StintUltraExpanded-Regular", identifier: "imgly_font_stint_ultra_expanded"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_sue_ellen_francisco.ttf",
                     name: "Sue Ellen Francisco",
                     identifier: "imgly_font_sue_ellen_francisco"),
            Font(displayName: "Trash Hand", fontName: "TrashHand", identifier: "imgly_font_trash_hand"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/ubuntu.ttf",
                     name: "Ubuntu",
                     identifier: "ubuntu"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_ultra.ttf",
                     name: "Ultra",
                     identifier: "imgly_font_ultra"),
            Font(displayName: "Vt323", fontName: "VT323-Regular", identifier: "imgly_font_vt323"),
            makeFont(url: "https://web-flix.web.app/assets/photoeditorsdk/font/fonts/imgly_font_wolesbro.ttf",
                     name: "Wolesbro",
                     identifier: "imgly_font_wolesbro"),
            Font(displayName: "Yeseva One", fontName: "YesevaOne-Regular", identifier: "imgly_font_yeseva_one"),
        ]
    }
    
    func makeFont(url: String, name: String, identifier: String) -> Font {
        return Font(url: URL(string: url)!, displayName: name, fontName: name, identifier: identifier)
    }
    
    fileprivate func createTitleToolbarLabel(title: String) -> UILabel {
        let padding = UIEdgeInsets(top: 0, left: 10, bottom: 0, right: 0)
        /// The right and left button has 60 width. 60 + 60 = 120 but the "save" button has a -20 left padding so 120 + 20 = 140
        let rect = CGRect(x: 0, y: 0, width: UIScreen.main.bounds.width - 140, height: 44).inset(by: padding)
        let label = UILabel(frame: rect)
        label.textAlignment = .left
        label.text = title
        label.textColor = .black
        label.backgroundColor = .white
        
        
        label.font = UIFont.systemFont(ofSize: 28, weight: .bold)
        
        return label
    }
    
    fileprivate func applyConstraintsOnTitleToolbarLabel(label: UILabel, view: UIView) {
        label.topAnchor.constraint(equalTo: view.topAnchor, constant: 0).isActive = true
        label.bottomAnchor.constraint(equalTo: view.bottomAnchor, constant: 0).isActive = true
        label.rightAnchor.constraint(equalTo: view.rightAnchor, constant: 0).isActive = true
        label.leftAnchor.constraint(equalTo: view.leftAnchor, constant: 0).isActive = true
    }
    
    static func preferredLocale() -> Locale {
        guard let preferredIdentifier = Locale.preferredLanguages.first else {
            return Locale.current
        }
        return Locale(identifier: preferredIdentifier)
    }
    
    fileprivate static func translate() {
        PESDK.localizationBlock = { stringToLocalize in
            return NSLocalizedString(stringToLocalize, tableName: "MyStrings", bundle: Bundle(for: SwiftPhotoEditorSdkPlugin.self), comment: "")
        }
    }
    
    private static let defaultTheme: Theme = {
        return Theme(backgroundColor: Colors.neutral50, menuBackgroundColor: .white, toolbarBackgroundColor: .white, primaryColor: Colors.primmary)
    }()
    
    private var theme = defaultTheme
}

extension SwiftPhotoEditorSdkPlugin: PhotoEditViewControllerDelegate {
    public func photoEditViewController(_ photoEditViewController: PhotoEditViewController, didSave image: UIImage, and data: Data) {
        if let navigationController = photoEditViewController.navigationController {
            navigationController.popViewController(animated: true)
        } else {
            if (data.count == 0) {
                globalResult(image.pngData())
            } else {
                globalResult(data)
            }
            
            controller.dismiss(animated: true, completion: nil)
        }
    }
    
    public func photoEditViewControllerDidFailToGeneratePhoto(_ photoEditViewController: PhotoEditViewController) {
        if let navigationController = photoEditViewController.navigationController {
            navigationController.popViewController(animated: true)
        } else {
            controller.dismiss(animated: true, completion: nil)
        }
    }
    
    public func photoEditViewControllerDidCancel(_ photoEditViewController: PhotoEditViewController) {
        if let navigationController = photoEditViewController.navigationController {
            navigationController.popViewController(animated: true)
        } else {
            controller.dismiss(animated: true, completion: nil)
        }
    }
}
